/*
 *     ParseTest.java  Nov 20, 2001
 *
 *     Katsuhisa Maruyama (maru@fse.cs.ritsumei.ac.jp)
 */

import jp.ac.ritsumei.cs.fse.jrt.parser.*;
import jp.ac.ritsumei.cs.fse.jrt.parser.summary.*;
import jp.ac.ritsumei.cs.fse.jrt.model.JavaFile;
import jp.ac.ritsumei.cs.fse.jrt.test.FileCompare;
import java.util.*;
import java.io.*;
import junit.framework.*;

public class ParseTest extends TestCase {
    public ParseTest(String name) {
        super(name);
    }

    public static void main(String args[]) {
        junit.textui.TestRunner.run(suite());
    }

    protected void setUp() {
        String JDKFilePath;
        if (System.getProperty("os.name").startsWith("Windows")) {
            JDKFilePath = "c:\\jdk1.3.1\\src.jar";
        } else {
            JDKFilePath = "/usr/local/j2sdk1_3_1/src.jar";
        }
        JavaModelFactory.getInstance().setJDKFile(JDKFilePath);
    }

    protected void tearDown() {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite();

        List testcases = new ArrayList();
        testcases.add(new String("./testcases/fowler"));
        // testcases.add(new String("./testcases/chart"));
        // testcases.add(new String("./testcases/tetris"));

        List jfiles = makeTestCases(testcases);
        Iterator it = jfiles.iterator();
        while (it.hasNext()) {
            final String name = (String)it.next();
            suite.addTest(new ParseTest("Parser Test" + name) {
                protected void runTest() {
                    testOutputFile(name);
                }
            });
        }
        return suite;
    }

    private static List makeTestCases(List testcases) {
        List jfiles = new ArrayList();
        Iterator it = testcases.iterator();
        while (it.hasNext()) {
            String dir = (String)it.next();
            Summary.getInstance().parse(dir);

            File directory = new File(dir);
            if (directory.isDirectory()) {
                String[] names = directory.list();
                for (int i = 0; i < names.length; i++) {
                    String name = dir + File.separator + names[i];
                    File file = new File(name);
                    if (file.isFile() && name.endsWith(".java")) {
                       jfiles.add(name);
                    }
                }
            }
        }
        return jfiles;
    }

    public void testOutputFile(String name) {
        JavaFile jfile = JavaModelFactory.getInstance().getEachJavaFile(name);

        File file = new File(name + ".code");
        ParseTestVisitor visitor = new ParseTestVisitor(file.getPath());
        jfile.accept(visitor);
        visitor.terminate();

        boolean result = FileCompare.compare(name, file.getPath());
        file.delete();

        assertTrue(result);
    }
}
